import ws from 'ws';
const WebSocket = global.WebSocket || ws;
export function isTradeUpdate(obj) {
    return (obj?.TradeUpdate?.trade !== undefined &&
        obj?.TradeUpdate?.status !== undefined);
}
function isStatusMessage(obj) {
    return obj.success !== undefined;
}
export class TradesFeed {
    _url;
    _socket;
    _connected;
    _reconnectionIntervalMs;
    _reconnectionMaxAttempts;
    _reconnectionAttempts;
    _onConnect = null;
    _onDisconnect = null;
    _onMessage = null;
    _onTrade = null;
    _onStatus = null;
    constructor(url, options) {
        this._url = url;
        this._reconnectionIntervalMs = options?.reconnectionIntervalMs ?? 5000;
        this._reconnectionMaxAttempts = options?.reconnectionMaxAttempts ?? -1;
        this._reconnectionAttempts = 0;
        this._connect();
        this._onMessage = (data) => {
            if (isTradeUpdate(data) && this._onTrade) {
                this._onTrade(data);
            }
            else if (isStatusMessage(data) && this._onStatus) {
                this._onStatus(data);
            }
        };
    }
    disconnect() {
        if (this._connected) {
            this._socket.close();
            this._connected = false;
        }
    }
    connected() {
        return this._connected;
    }
    onConnect(callback) {
        this._onConnect = callback;
    }
    onDisconnect(callback) {
        this._onDisconnect = callback;
    }
    onMessage(callback) {
        this._onMessage = callback;
    }
    onTrade(callback) {
        this._onTrade = callback;
    }
    onStatus(callback) {
        this._onStatus = callback;
    }
    subscribe(subscriptions) {
        if (this.connected()) {
            this._socket.send(JSON.stringify({
                command: 'subscribe',
                ...subscriptions,
            }));
        }
        else {
            console.warn('[TradesFeed] attempt to subscribe when not connected');
        }
    }
    unsubscribe(params) {
        if (this.connected()) {
            this._socket.send(JSON.stringify({
                command: 'unsubscribe',
                ...params,
            }));
        }
        else {
            console.warn('[TradesFeed] attempt to unsubscribe when not connected');
        }
    }
    _connect() {
        this._socket = new WebSocket(this._url);
        this._socket.addEventListener('error', (err) => {
            console.warn(`[MangoFeed] connection error: ${err.message}`);
            if (this._reconnectionAttemptsExhausted()) {
                console.error('[MangoFeed] fatal connection error');
                throw err.error;
            }
        });
        this._socket.addEventListener('open', () => {
            this._connected = true;
            this._reconnectionAttempts = 0;
            if (this._onConnect)
                this._onConnect();
        });
        this._socket.addEventListener('close', () => {
            this._connected = false;
            setTimeout(() => {
                if (!this._reconnectionAttemptsExhausted()) {
                    this._reconnectionAttempts++;
                    this._connect();
                }
            }, this._reconnectionIntervalMs);
            if (this._onDisconnect)
                this._onDisconnect(this._reconnectionAttemptsExhausted());
        });
        this._socket.addEventListener('message', (msg) => {
            try {
                const data = JSON.parse(msg.data);
                if (isStatusMessage(data) && this._onStatus) {
                    this._onStatus(data);
                }
                else if (this._onMessage) {
                    this._onMessage(data);
                }
            }
            catch (err) {
                console.warn('[MangoFeed] error deserializing message', err);
            }
        });
    }
    _reconnectionAttemptsExhausted() {
        return (this._reconnectionMaxAttempts != -1 &&
            this._reconnectionAttempts >= this._reconnectionMaxAttempts);
    }
}
